package org.eclipse.stardust.embedded;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationContext;
import org.springframework.test.AbstractSingleSpringContextTests;
import org.eclipse.stardust.embedded.util.Util;
import org.eclipse.stardust.engine.api.runtime.AdministrationService;
import org.eclipse.stardust.engine.api.runtime.DocumentManagementService;
import org.eclipse.stardust.engine.api.runtime.QueryService;
import org.eclipse.stardust.engine.api.runtime.ServiceFactory;
import org.eclipse.stardust.engine.api.runtime.ServiceFactoryLocator;
import org.eclipse.stardust.engine.api.runtime.UserService;
import org.eclipse.stardust.engine.api.runtime.WorkflowService;
import org.eclipse.stardust.engine.api.spring.SpringUtils;

/**
 * @Autoinject with @Qualifier on Fields is supported
 * 
 * @author 
 *
 */

public abstract class AbstractGeneralIppContextTest extends AbstractSingleSpringContextTests  {
	private final Object LOCK = new Object();
	private volatile ServiceFactory serviceFactory;

	
	protected void setupUserGrants(UserService pUserService) {
	}

	@Override
	protected final void prepareTestInstance() throws Exception {
		injectBeans(getClass(), getApplicationContext());
		cleanupRuntime(true);
		deployModel(getProcessModelFile());
		setupUserGrants(getUserService());
	}

	@SuppressWarnings("deprecation")
	private void deployModel(File pProcessModelFile) throws IOException {
		getAdministrationService().deployModel(Util.loadFromFile(pProcessModelFile), 0);
	}
	
	/**
	 * Wrapper for {@link AdministrationService#cleanupRuntime(boolean)}
	 * @param keepUsers 
	 */
	protected void cleanupRuntime(boolean keepUsers) {
		getAdministrationService().cleanupRuntime(keepUsers);
		SpringUtils.setApplicationContext(getApplicationContext());
	}

	protected AdministrationService getAdministrationService() {
		return getServiceFactory().getAdministrationService();
	}
	
	protected abstract File getProcessModelFile();
	
	protected void cleanupRuntimeAndModels() {
		getAdministrationService().cleanupRuntimeAndModels();
		SpringUtils.setApplicationContext(getApplicationContext());
	}
	
	protected DocumentManagementService getDocumentManagementService() {
		return getServiceFactory().getDocumentManagementService();
	}
	
	protected UserService getUserService() {
		return getServiceFactory().getUserService();
	}
	
	protected QueryService getQueryService() {
		return getServiceFactory().getQueryService();
	}
	protected WorkflowService getWorkflowService() {
		return getServiceFactory().getWorkflowService();
	}

	protected abstract String getPassword();
	
	private final ServiceFactory getServiceFactory() {
		if (serviceFactory == null) {
			synchronized (LOCK) {
				if (serviceFactory == null) {
					serviceFactory = ServiceFactoryLocator.get(getUsername(), getPassword());
				}	
			}
		}
		return serviceFactory;
	}

	protected abstract String getUsername();
	
	private void injectBeans(Class<?> pClass, ApplicationContext applicationContext) throws IllegalArgumentException, IllegalAccessException {
		if (pClass.equals(AbstractEmbeddedIppContextTest.class)) {
			return;
		}
		for (Field aField : pClass.getDeclaredFields()) {
			Autowired lAutowired = aField.getAnnotation(Autowired.class);
			if (lAutowired != null) {
				Qualifier lQualifier = aField.getAnnotation(Qualifier.class);
				String lBeanName;
				if (lQualifier != null) {
					lBeanName = lQualifier.value();
				} else {
					lBeanName = aField.getName();
				}
				
				Object lBean = applicationContext.getBean(lBeanName);
				aField.setAccessible(true);
				aField.set(this, lBean);
			}
		}
	}

}
