package org.eclipse.stardust.embedded;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.StringTokenizer;

import javax.sql.DataSource;

import org.eclipse.stardust.embedded.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;


public class AuditTrailInitalizer implements InitializingBean {
	private static final Logger logger = LoggerFactory.getLogger(AuditTrailInitalizer.class);
	private DataSource dataSource;
	private String auditTrailDdlFile;
	
	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	public void setAuditTrailDdlFile(String auditTrailDdlFile) {
		this.auditTrailDdlFile = auditTrailDdlFile;
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		try {
			setupAuditTrail(this.dataSource, getAuditTrailDdlFile());
				
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		} catch (IOException e) {
			throw new RuntimeException(e.getMessage(), e);
		} 
	}
	
	protected File getAuditTrailDdlFile() {
		return new File(this.auditTrailDdlFile);
	}
	
	private void setupAuditTrail(DataSource ds, File pAuditTrailDdlFile) throws SQLException, IOException {
		Connection con = ds.getConnection();
		try {
			Statement stmt = con.createStatement();
			
			try {
				stmt.execute("CREATE SCHEMA carnot");
			} catch (SQLException se) {
				if (30000 == se.getErrorCode()) {
					logger.info("Schema already exists! Delete the Derby DB at all to start from scratch.");
					return;
				}
				throw se;
			}
			logger.info("AudiTrail created");
			
			StringTokenizer st = new StringTokenizer(Util.loadFromFile(pAuditTrailDdlFile), ";");
			while(st.hasMoreTokens()) {
				String aSql = st.nextToken().trim();
				//System.out.println("==== " + aSql);
				if (aSql.length() > 0) {
					stmt.executeUpdate(aSql);
				}
			}
			stmt.close();
			con.commit();
		} finally {
			con.close();
		}
		
		logger.info("AudiTrail initalized");
	}
}
