CREATE TABLE activity_instance (oid BIGINT GENERATED ALWAYS AS IDENTITY, state INTEGER, startTime BIGINT, lastModificationTime BIGINT, model BIGINT, activity BIGINT, currentPerformer BIGINT, currentUserPerformer BIGINT, performedBy BIGINT, currentDepartment BIGINT, criticality DOUBLE, propertiesAvailable INTEGER, processInstance BIGINT);
CREATE UNIQUE INDEX activity_inst_idx1 ON activity_instance(oid);
CREATE INDEX activity_inst_idx2 ON activity_instance(currentPerformer, currentDepartment);
CREATE INDEX activity_inst_idx3 ON activity_instance(currentUserPerformer, currentPerformer, currentDepartment);
CREATE INDEX activity_inst_idx4 ON activity_instance(performedBy);
CREATE INDEX activity_inst_idx5 ON activity_instance(processInstance);
CREATE INDEX activity_inst_idx6 ON activity_instance(startTime, activity, state);
CREATE INDEX activity_inst_idx7 ON activity_instance(state);
CREATE INDEX activity_inst_idx8 ON activity_instance(activity, processInstance);
CREATE INDEX activity_inst_idx9 ON activity_instance(criticality, processInstance);

CREATE TABLE activity_instance_lck (oid BIGINT);
CREATE UNIQUE INDEX act_inst_lck_idx ON activity_instance_lck(oid);

CREATE TABLE activity_inst_log (oid BIGINT GENERATED ALWAYS AS IDENTITY, type INTEGER, stamp BIGINT, model BIGINT, participant BIGINT, workflowUser BIGINT, activityInstance BIGINT);
CREATE INDEX act_inst_log_idx1 ON activity_inst_log(activityInstance);
CREATE UNIQUE INDEX act_inst_log_idx2 ON activity_inst_log(oid);

CREATE TABLE act_inst_history (processInstance BIGINT, activityInstance BIGINT, state INTEGER, fromTimestamp BIGINT, untilTimestamp BIGINT, domain BIGINT, performerKind INTEGER, performer BIGINT, department BIGINT, onBehalfOfKind INTEGER, onBehalfOf BIGINT, onBehalfOfDepartment BIGINT, onBehalfOfUser BIGINT, workflowUser BIGINT);
CREATE UNIQUE INDEX act_inst_hist_idx1 ON act_inst_history(activityInstance, fromTimestamp);
CREATE INDEX act_inst_hist_idx2 ON act_inst_history(processInstance);

CREATE TABLE act_inst_property (oid BIGINT GENERATED ALWAYS AS IDENTITY, objectOID BIGINT, name VARCHAR(300), type_key INTEGER, number_value BIGINT, string_value VARCHAR(128), lastModificationTime BIGINT, workflowUser BIGINT);
CREATE INDEX act_inst_prp_idx1 ON act_inst_property(objectOID);
CREATE INDEX act_inst_prp_idx2 ON act_inst_property(type_key, number_value);
CREATE INDEX act_inst_prp_idx3 ON act_inst_property(type_key, string_value);
CREATE UNIQUE INDEX act_inst_prp_idx4 ON act_inst_property(oid);

CREATE TABLE trans_inst (oid BIGINT GENERATED ALWAYS AS IDENTITY, model BIGINT, transition BIGINT, processInstance BIGINT, source BIGINT, target BIGINT);
CREATE INDEX trans_inst_idx1 ON trans_inst(processInstance);
CREATE UNIQUE INDEX trans_inst_idx2 ON trans_inst(oid);

CREATE TABLE trans_token (oid BIGINT GENERATED ALWAYS AS IDENTITY, processInstance BIGINT, model BIGINT, transition BIGINT, source BIGINT, target BIGINT, isConsumed INTEGER);
CREATE INDEX trans_token_idx1 ON trans_token(processInstance);
CREATE UNIQUE INDEX trans_token_idx2 ON trans_token(oid);
CREATE INDEX trans_token_idx3 ON trans_token(processInstance, transition, model, isConsumed);

CREATE TABLE trans_token_lck (oid BIGINT);
CREATE UNIQUE INDEX trans_tok_lck_idx ON trans_token_lck(oid);

CREATE TABLE daemon_log (oid BIGINT GENERATED ALWAYS AS IDENTITY, type VARCHAR(100), code INTEGER, stamp BIGINT, state INTEGER, partition BIGINT);
CREATE UNIQUE INDEX daemon_log_idx1 ON daemon_log(oid);
CREATE INDEX daemon_log_idx2 ON daemon_log(code, partition, type);

CREATE TABLE data_value (oid BIGINT GENERATED ALWAYS AS IDENTITY, model BIGINT, data BIGINT, string_value VARCHAR(128), number_value BIGINT, double_value DOUBLE, type_key INTEGER, processInstance BIGINT);
CREATE UNIQUE INDEX data_values_index1 ON data_value(oid);
CREATE INDEX data_values_index2 ON data_value(processInstance);
CREATE INDEX data_values_index3 ON data_value(type_key);
CREATE INDEX data_values_index4 ON data_value(number_value);
CREATE INDEX data_values_index5 ON data_value(string_value);
CREATE UNIQUE INDEX data_values_index6 ON data_value(data, processInstance);

CREATE TABLE data_value_lck (oid BIGINT);
CREATE UNIQUE INDEX data_value_lck_idx ON data_value_lck(oid);

CREATE TABLE event_binding (oid BIGINT GENERATED ALWAYS AS IDENTITY, objectOID BIGINT, type INTEGER, model BIGINT, handlerOID BIGINT, bindStamp BIGINT, targetStamp BIGINT, partition BIGINT);
CREATE UNIQUE INDEX event_binding_idx1 ON event_binding(oid);
CREATE INDEX event_binding_idx2 ON event_binding(objectOID, type, handlerOID, model, partition);
CREATE INDEX event_binding_idx3 ON event_binding(targetStamp, partition);

CREATE TABLE log_entry (oid BIGINT GENERATED ALWAYS AS IDENTITY, type INTEGER, code INTEGER, subject VARCHAR(300), stamp BIGINT, processInstance BIGINT, activityInstance BIGINT, workflowUser BIGINT, partition BIGINT);
CREATE UNIQUE INDEX log_entry_idx1 ON log_entry(oid);
CREATE INDEX log_entry_idx2 ON log_entry(activityInstance);
CREATE INDEX log_entry_idx3 ON log_entry(processInstance);

CREATE TABLE property (oid BIGINT GENERATED ALWAYS AS IDENTITY, name VARCHAR(300), value VARCHAR(500), locale VARCHAR(300), flags INTEGER, partition BIGINT);
CREATE UNIQUE INDEX property_idx1 ON property(oid);

CREATE TABLE timer_log (oid BIGINT GENERATED ALWAYS AS IDENTITY, model BIGINT, triggerOID BIGINT, stamp BIGINT);
CREATE UNIQUE INDEX timer_log_idx1 ON timer_log(oid);

CREATE TABLE usergroup (oid BIGINT GENERATED ALWAYS AS IDENTITY, id VARCHAR(50), name VARCHAR(150), validFrom BIGINT, validTo BIGINT, description VARCHAR(300), partition BIGINT);
CREATE UNIQUE INDEX usergroup_idx1 ON usergroup(oid);
CREATE UNIQUE INDEX usergroup_idx2 ON usergroup(id, partition);

CREATE TABLE usergroup_property (oid BIGINT GENERATED ALWAYS AS IDENTITY, objectOID BIGINT, name VARCHAR(300), type_key INTEGER, number_value BIGINT, string_value VARCHAR(128), lastModificationTime BIGINT, workflowUser BIGINT);
CREATE INDEX usergrp_prp_idx1 ON usergroup_property(objectOID);
CREATE INDEX usergrp_prp_idx2 ON usergroup_property(type_key, number_value);
CREATE INDEX usergrp_prp_idx3 ON usergroup_property(type_key, string_value);
CREATE UNIQUE INDEX usergrp_prp_idx4 ON usergroup_property(oid);

CREATE TABLE workflowuser (oid BIGINT GENERATED ALWAYS AS IDENTITY, account VARCHAR(50), firstName VARCHAR(150), lastName VARCHAR(150), password VARCHAR(300), eMail VARCHAR(150), validFrom BIGINT, validTo BIGINT, description VARCHAR(300), failedLoginCount BIGINT, lastLoginTime BIGINT, extendedState INTEGER, realm BIGINT);
CREATE UNIQUE INDEX workflowuser_idx1 ON workflowuser(oid);
CREATE UNIQUE INDEX workflowuser_idx2 ON workflowuser(account, realm);

CREATE TABLE user_property (oid BIGINT GENERATED ALWAYS AS IDENTITY, objectOID BIGINT, name VARCHAR(300), type_key INTEGER, number_value BIGINT, string_value VARCHAR(128), lastModificationTime BIGINT, scope VARCHAR(300));
CREATE INDEX user_prp_idx1 ON user_property(objectOID);
CREATE INDEX user_prp_idx2 ON user_property(type_key, number_value);
CREATE INDEX user_prp_idx3 ON user_property(type_key, string_value);
CREATE UNIQUE INDEX user_prp_idx4 ON user_property(oid);

CREATE TABLE wfuser_session (oid BIGINT GENERATED ALWAYS AS IDENTITY, workflowUser BIGINT, clientId VARCHAR(300), startTime BIGINT, lastModificationTime BIGINT, expirationTime BIGINT);
CREATE UNIQUE INDEX wfusr_session_idx1 ON wfuser_session(oid);
CREATE INDEX wfusr_session_idx2 ON wfuser_session(workflowUser, startTime);

CREATE TABLE user_participant (oid BIGINT GENERATED ALWAYS AS IDENTITY, participant BIGINT, department BIGINT, onBehalfOf BIGINT, workflowUser BIGINT);
CREATE INDEX user_particip_idx1 ON user_participant(workflowUser);
CREATE INDEX user_particip_idx2 ON user_participant(participant, department);
CREATE UNIQUE INDEX user_particip_idx3 ON user_participant(oid);

CREATE TABLE user_usergroup (oid BIGINT GENERATED ALWAYS AS IDENTITY, workflowUser BIGINT, userGroup BIGINT);
CREATE UNIQUE INDEX user_usergrp_idx1 ON user_usergroup(oid);
CREATE INDEX user_usergrp_idx2 ON user_usergroup(workflowUser);
CREATE INDEX user_usergrp_idx3 ON user_usergroup(userGroup);

CREATE TABLE process_instance (oid BIGINT GENERATED ALWAYS AS IDENTITY, startTime BIGINT, terminationTime BIGINT, state INTEGER, model BIGINT, processDefinition BIGINT, priority INTEGER, deployment BIGINT, tokenCount BIGINT, propertiesAvailable INTEGER, rootProcessInstance BIGINT, scopeProcessInstance BIGINT, startingUser BIGINT, startingActivityInstance BIGINT);
CREATE UNIQUE INDEX proc_inst_idx1 ON process_instance(oid);
CREATE INDEX proc_inst_idx2 ON process_instance(state, terminationTime, processDefinition);
CREATE INDEX proc_inst_idx3 ON process_instance(state, startTime, terminationTime);
CREATE INDEX proc_inst_idx4 ON process_instance(startingActivityInstance);
CREATE INDEX proc_inst_idx5 ON process_instance(rootProcessInstance, state, oid);
CREATE INDEX proc_inst_idx6 ON process_instance(scopeProcessInstance, state, oid);

CREATE TABLE process_instance_lck (oid BIGINT);
CREATE UNIQUE INDEX proc_inst_lck_idx ON process_instance_lck(oid);

CREATE TABLE proc_inst_property (oid BIGINT GENERATED ALWAYS AS IDENTITY, objectOID BIGINT, name VARCHAR(300), type_key INTEGER, number_value BIGINT, string_value VARCHAR(128), lastModificationTime BIGINT, workflowUser BIGINT);
CREATE INDEX proc_inst_prp_idx1 ON proc_inst_property(objectOID);
CREATE INDEX proc_inst_prp_idx2 ON proc_inst_property(type_key, number_value);
CREATE INDEX proc_inst_prp_idx3 ON proc_inst_property(type_key, string_value);
CREATE UNIQUE INDEX proc_inst_prp_idx4 ON proc_inst_property(oid);

CREATE TABLE procinst_scope (processInstance BIGINT, scopeProcessInstance BIGINT, rootProcessInstance BIGINT);
CREATE UNIQUE INDEX procinst_scope_i1 ON procinst_scope(processInstance, scopeProcessInstance);
CREATE UNIQUE INDEX procinst_scope_i2 ON procinst_scope(scopeProcessInstance, processInstance);
CREATE INDEX procinst_scope_i3 ON procinst_scope(rootProcessInstance);

CREATE TABLE procinst_hierarchy (processInstance BIGINT, subProcessInstance BIGINT);
CREATE UNIQUE INDEX procinst_hier_idx1 ON procinst_hierarchy(processInstance, subProcessInstance);
CREATE UNIQUE INDEX procinst_hier_idx2 ON procinst_hierarchy(subProcessInstance, processInstance);

CREATE TABLE structured_data_value (oid BIGINT GENERATED ALWAYS AS IDENTITY, parent BIGINT, entryKey VARCHAR(50), xpath BIGINT, type_key INTEGER, string_value VARCHAR(128), number_value BIGINT, double_value DOUBLE, processInstance BIGINT);
CREATE UNIQUE INDEX struct_dv_index1 ON structured_data_value(oid);
CREATE INDEX struct_dv_index2 ON structured_data_value(parent);
CREATE INDEX struct_dv_index3 ON structured_data_value(xpath);
CREATE INDEX struct_dv_index4 ON structured_data_value(type_key);
CREATE INDEX struct_dv_index5 ON structured_data_value(number_value);
CREATE INDEX struct_dv_index6 ON structured_data_value(string_value);
CREATE INDEX struct_dv_index7 ON structured_data_value(processInstance);

CREATE TABLE structured_data_value_lck (oid BIGINT);
CREATE UNIQUE INDEX struct_dv_lck_idx ON structured_data_value_lck(oid);

CREATE TABLE domain (oid BIGINT GENERATED ALWAYS AS IDENTITY, id VARCHAR(50), description VARCHAR(4000), partition BIGINT, superDomain BIGINT);
CREATE UNIQUE INDEX domain_idx1 ON domain(oid);
CREATE UNIQUE INDEX domain_idx2 ON domain(id, partition);

CREATE TABLE domain_hierarchy (oid BIGINT GENERATED ALWAYS AS IDENTITY, superDomain BIGINT, subDomain BIGINT);
CREATE UNIQUE INDEX domain_hier_idx1 ON domain_hierarchy(oid);
CREATE UNIQUE INDEX domain_hier_idx2 ON domain_hierarchy(superDomain, subDomain);

CREATE TABLE wfuser_domain (oid BIGINT GENERATED ALWAYS AS IDENTITY, validFrom BIGINT, validTo BIGINT, domain BIGINT, wfuser BIGINT);
CREATE UNIQUE INDEX wfuser_domain_idx1 ON wfuser_domain(oid);
CREATE UNIQUE INDEX wfuser_domain_idx2 ON wfuser_domain(domain, wfuser);

CREATE TABLE wfuser_realm (oid BIGINT GENERATED ALWAYS AS IDENTITY, id VARCHAR(50), name VARCHAR(100), description VARCHAR(4000), partition BIGINT);
CREATE UNIQUE INDEX wfuser_realm_idx1 ON wfuser_realm(oid);
CREATE UNIQUE INDEX wfuser_realm_idx2 ON wfuser_realm(id, partition);

CREATE TABLE workitem (activityInstance BIGINT, processInstance BIGINT, scopeProcessInstance BIGINT, rootProcessInstance BIGINT, model BIGINT, activity BIGINT, state INTEGER, startTime BIGINT, lastModificationTime BIGINT, domain BIGINT, criticality DOUBLE, performerKind INTEGER, performer BIGINT, department BIGINT);
CREATE UNIQUE INDEX workitem_idx1 ON workitem(activityInstance);
CREATE INDEX workitem_idx2 ON workitem(performer, department, performerKind, state);
CREATE INDEX workitem_idx3 ON workitem(state);
CREATE INDEX workitem_idx4 ON workitem(processInstance);
CREATE INDEX workitem_idx5 ON workitem(scopeProcessInstance);

CREATE TABLE structured_data (oid BIGINT, xpath VARCHAR(200), data BIGINT, model BIGINT);
CREATE UNIQUE INDEX struct_data_idx1 ON structured_data(oid, model);
CREATE INDEX struct_data_idx2 ON structured_data(xpath);

CREATE TABLE clob_data (oid BIGINT GENERATED ALWAYS AS IDENTITY, ownerId BIGINT, ownerType VARCHAR(32), stringValue CLOB);
CREATE INDEX clob_dt_i1 ON clob_data(ownerId, ownerType);
CREATE UNIQUE INDEX clob_dt_i2 ON clob_data(oid);

CREATE TABLE department (oid BIGINT GENERATED ALWAYS AS IDENTITY, id VARCHAR(50), name VARCHAR(150), description VARCHAR(4000), organization BIGINT, parentDepartment BIGINT, partition BIGINT);
CREATE UNIQUE INDEX department_idx1 ON department(oid);
CREATE UNIQUE INDEX department_idx2 ON department(id, organization, parentDepartment);

CREATE TABLE department_hierarchy (superDepartment BIGINT, subDepartment BIGINT);
CREATE UNIQUE INDEX department_hier_idx1 ON department_hierarchy(superDepartment, subDepartment);
CREATE UNIQUE INDEX department_hier_idx2 ON department_hierarchy(subDepartment, superDepartment);

CREATE TABLE model_ref (code INTEGER, modelOid BIGINT, id VARCHAR(300), refOid BIGINT, deployment BIGINT);
CREATE UNIQUE INDEX model_ref_idx1 ON model_ref(code, modelOid, id, deployment);
CREATE UNIQUE INDEX model_ref_idx2 ON model_ref(code, modelOid, refOid, deployment);

CREATE TABLE model_dep (oid BIGINT GENERATED ALWAYS AS IDENTITY, deployer BIGINT, deploymentTime BIGINT, validFrom BIGINT, deploymentComment VARCHAR(300));
CREATE UNIQUE INDEX model_dep_idx1 ON model_dep(oid);
CREATE INDEX model_dep_idx2 ON model_dep(deploymentTime);
CREATE INDEX model_dep_idx3 ON model_dep(deployer);

CREATE TABLE model_dep_lck (oid BIGINT);
CREATE UNIQUE INDEX model_dep_lck_idx ON model_dep_lck(oid);

CREATE TABLE preferences (ownerId BIGINT, ownerType VARCHAR(32), moduleId VARCHAR(255), preferencesId VARCHAR(255), partition BIGINT, stringValue CLOB);
CREATE UNIQUE INDEX preferences_idx1 ON preferences(ownerId, ownerType, moduleId, preferencesId, partition);

CREATE TABLE procinst_link (processInstance BIGINT, linkedProcessInstance BIGINT, linkType BIGINT, createTime BIGINT, creatingUser BIGINT, linkingComment VARCHAR(255));

CREATE TABLE link_type (oid BIGINT GENERATED ALWAYS AS IDENTITY, id VARCHAR(50), description VARCHAR(255), partition BIGINT);
CREATE UNIQUE INDEX link_type_idx1 ON link_type(oid);

CREATE TABLE model (oid BIGINT GENERATED ALWAYS AS IDENTITY, id VARCHAR(50), name VARCHAR(300), validFrom BIGINT, validTo BIGINT, predecessor BIGINT, deploymentComment VARCHAR(300), deploymentStamp BIGINT, version VARCHAR(300), revision BIGINT, disabled BIGINT, partition BIGINT);
CREATE UNIQUE INDEX model_idx1 ON model(oid);

CREATE TABLE STRING_DATA (oid BIGINT GENERATED ALWAYS AS IDENTITY, objectid BIGINT, data_type VARCHAR(32), data VARCHAR(4000));
CREATE INDEX str_dt_i1 ON STRING_DATA(objectid, data_type);
CREATE UNIQUE INDEX str_dt_i2 ON STRING_DATA(oid);

CREATE TABLE activity (oid BIGINT, model BIGINT, id VARCHAR(50), name VARCHAR(100), processDefinition BIGINT, description VARCHAR(4000));
CREATE UNIQUE INDEX activity_idx1 ON activity(oid, model);
CREATE INDEX activity_idx2 ON activity(id, oid, model);

CREATE TABLE data (oid BIGINT, model BIGINT, id VARCHAR(50), name VARCHAR(100), description VARCHAR(4000));
CREATE UNIQUE INDEX data_idx1 ON data(oid, model);
CREATE INDEX data_idx2 ON data(id, oid, model);

CREATE TABLE event_handler (oid BIGINT, model BIGINT, id VARCHAR(50), name VARCHAR(100), processDefinition BIGINT, activity BIGINT);
CREATE UNIQUE INDEX event_handler_idx1 ON event_handler(oid, model);
CREATE INDEX event_handler_idx2 ON event_handler(id, oid, model);

CREATE TABLE participant (oid BIGINT, model BIGINT, id VARCHAR(50), name VARCHAR(100), type INTEGER, description VARCHAR(4000));
CREATE UNIQUE INDEX participant_idx1 ON participant(oid, model);
CREATE INDEX participant_idx2 ON participant(id, oid, model);

CREATE TABLE partition (oid BIGINT GENERATED ALWAYS AS IDENTITY, id VARCHAR(50), description VARCHAR(4000));
CREATE UNIQUE INDEX partition_idx1 ON partition(oid);
CREATE UNIQUE INDEX partition_idx2 ON partition(id);

CREATE TABLE partition_lck (oid BIGINT);
CREATE UNIQUE INDEX partition_lck_idx ON partition_lck(oid);

CREATE TABLE process_definition (oid BIGINT, model BIGINT, id VARCHAR(50), name VARCHAR(100), description VARCHAR(4000));
CREATE UNIQUE INDEX proc_def_idx1 ON process_definition(oid, model);
CREATE INDEX proc_def_idx2 ON process_definition(id, oid, model);

CREATE TABLE transition (oid BIGINT, model BIGINT, id VARCHAR(50), processDefinition BIGINT, sourceActivity BIGINT, targetActivity BIGINT, condition VARCHAR(200));
CREATE UNIQUE INDEX trans_idx1 ON transition(oid, model);
CREATE INDEX trans_idx2 ON transition(id, oid, model);

CREATE TABLE process_trigger (oid BIGINT, model BIGINT, id VARCHAR(50), name VARCHAR(100), processDefinition BIGINT);
CREATE UNIQUE INDEX proc_trigger_idx1 ON process_trigger(oid, model);
CREATE INDEX proc_trigger_idx2 ON process_trigger(id, oid, model);



INSERT INTO property (partition,name,value,locale,flags) VALUES (-1, 'sysop.password', 'sysop', 'DEFAULT', 0)
;
INSERT INTO property (partition,name,value,locale,flags) VALUES (-1, 'carnot.version', '8.1.0', 'DEFAULT', 0)
;
INSERT INTO partition (id,description) VALUES ('default', NULL)
;
INSERT INTO partition_lck (oid) SELECT p.oid FROM partition p WHERE p.id = 'default'
;
INSERT INTO domain (id,partition,superDomain,description) SELECT 'default', p.oid, cast(NULL as bigint), cast(NULL as varchar(1)) FROM partition p WHERE p.id = 'default'
;
INSERT INTO domain_hierarchy (subDomain,superDomain) SELECT d.oid, d.oid FROM domain d WHERE d.id = 'default'
;
INSERT INTO carnot.wfuser_realm (id,partition,name,description) SELECT 'carnot', p.oid, 'CARNOT', cast(NULL as varchar(1)) FROM partition p WHERE p.id = 'default'
;
INSERT INTO carnot.workflowuser (account,firstName,lastName,password,eMail,validFrom,validTo,description,failedLoginCount,lastLoginTime,realm) SELECT 'motu', 'Master', 'Of the Universe', 'motu', cast(NULL as varchar(1)), 1288805787551, 0, cast(NULL as varchar(1)), 0, 0, r.oid FROM wfuser_realm r WHERE r.id = 'carnot'
;
INSERT INTO link_type (id,description,partition) SELECT 'switch', 'Peer Process Instance', p.oid FROM partition p WHERE p.id = 'default'
;
INSERT INTO link_type (id,description,partition) SELECT 'join', 'Join Process Instance', p.oid FROM partition p WHERE p.id = 'default'
;
INSERT INTO link_type (id,description,partition) SELECT 'upgrade', 'Upgrade Process Instance', p.oid FROM partition p WHERE p.id = 'default';

